/* BdfServer_JsonProducers - Copyright (c) 2022-2023 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.jsonproducers.selection;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfCommandResult;
import fr.exemole.bdfserver.api.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.api.instruction.BdfParameters;
import fr.exemole.bdfserver.api.instruction.OutputParameters;
import fr.exemole.bdfserver.api.interaction.InteractionConstants;
import fr.exemole.bdfserver.api.interaction.domains.SelectionDomain;
import fr.exemole.bdfserver.jsonproducers.ResultJsonProducer;
import fr.exemole.bdfserver.tools.BdfServerUtils;
import fr.exemole.bdfserver.tools.instruction.BdfErrors;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.subsettree.TreeUtils;
import net.fichotheque.corpus.Fiches;
import net.fichotheque.selection.FichothequeQueries;
import net.fichotheque.selection.SelectionContext;
import net.fichotheque.selection.SelectionDef;
import net.fichotheque.tools.corpus.FichesBuilder;
import net.fichotheque.tools.permission.PermissionUtils;
import net.fichotheque.utils.selection.FicheSelectorBuilder;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public final class SelectionJsonProducerFactory {

    private SelectionJsonProducerFactory() {

    }

    public static JsonProducer getJsonProducer(OutputParameters parameters) throws ErrorMessageException {
        String json = parameters.getOutput();
        BdfCommandResult bdfCommandResult = parameters.getBdfCommandResult();
        BdfServer bdfServer = parameters.getBdfServer();
        RequestMap requestMap = parameters.getRequestMap();
        Lang lang = parameters.getWorkingLang();
        ResultJsonProducer jsonProducer = new ResultJsonProducer(bdfCommandResult, parameters.getMessageLocalisation());
        switch (json) {
            case SelectionDomain.FQL_JSON: {
                boolean done = false;
                if (bdfCommandResult != null) {
                    FichothequeQueries fichothequeQueries = (FichothequeQueries) bdfCommandResult.getResultObject(BdfInstructionConstants.FICHOTHEQUEQUERIES_OBJ);
                    if (fichothequeQueries != null) {
                        done = true;
                        jsonProducer.add(new FqlJsonProperty(parameters.getFichotheque(), fichothequeQueries));
                    }
                }
                if (!done) {
                    throw BdfErrors.missingCommandResultJson(SelectionDomain.FQL_JSON);
                }
                break;
            }
            case SelectionDomain.FICHES_JSON: {
                Fiches fiches = getFiches(parameters, parameters.getRequestMap());
                jsonProducer.add(new FichesJsonProperty(parameters.getPermissionSummary(), fiches, BdfInstructionUtils.getCellEngine(parameters, parameters.getRequestMap()), parameters.getWorkingLang()));
                break;
            }
            case SelectionDomain.SELECTIONDEF_JSON: {
                jsonProducer.add(new SelectionDefJsonProperty(getSelectionDef(bdfServer, requestMap, bdfCommandResult), lang));
                break;
            }
            case SelectionDomain.SELECTIONDEF_ARRAY_JSON: {
                jsonProducer.add(new SelectionDefArrayJsonProperty(bdfServer, lang));
                break;
            }
        }
        if (jsonProducer.isEmpty()) {
            return null;
        } else {
            return jsonProducer;
        }
    }

    private static Fiches getFiches(BdfParameters bdfParameters, RequestMap requestMap) throws ErrorMessageException {
        BdfServer bdfServer = bdfParameters.getBdfServer();
        SelectionContext selectionContext = BdfServerUtils.initSelectionContextBuilder(bdfParameters)
                .setSubsetAccessPredicate(bdfParameters.getPermissionSummary().getSubsetAccessPredicate())
                .setFichePredicate(PermissionUtils.getFichePredicate(bdfParameters.getPermissionSummary()))
                .toSelectionContext();
        FicheSelectorBuilder builder = FicheSelectorBuilder.init(selectionContext);
        String selection = requestMap.getParameter(InteractionConstants.SELECTION_PARAMNAME);
        if (selection != null) {
            BdfInstructionUtils.populateFromSelection(bdfServer, requestMap, builder);
        } else {
            BdfInstructionUtils.populateFromXml(bdfServer, requestMap, builder);
        }
        return FichesBuilder.init(BdfInstructionUtils.getComparator(selectionContext.getWorkingLang(), requestMap))
                .initSubsetKeyOrder(TreeUtils.getCorpusKeyList(bdfParameters.getBdfServer()))
                .populate(builder.toFicheSelector())
                .toFiches();
    }


    private static SelectionDef getSelectionDef(BdfServer bdfServer, RequestMap requestMap, BdfCommandResult bdfCommandResult) throws ErrorMessageException {
        SelectionDef selectionDefDef = null;
        if (bdfCommandResult != null) {
            selectionDefDef = (SelectionDef) bdfCommandResult.getResultObject(BdfInstructionConstants.SELECTIONDEF_OBJ);
        }
        if (selectionDefDef == null) {
            selectionDefDef = BdfInstructionUtils.getMandatorySelectionDef(bdfServer.getSelectionManager(), requestMap);
        }
        return selectionDefDef;
    }

}
